/**
 * @overview WMF Command Manager
 * @author Jan "Waffle" Christophersen
 * @copyright (c) Jan "Waffle" Christophersen
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

var _get = function get(_x, _x2, _x3) { var _again = true; _function: while (_again) { var object = _x, property = _x2, receiver = _x3; _again = false; if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { _x = parent; _x2 = property; _x3 = receiver; _again = true; desc = parent = undefined; continue _function; } } else if ('value' in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } } };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _wmfModule = require('../../wmfModule');

var _wmfModule2 = _interopRequireDefault(_wmfModule);

var _wmfCommand = require('./wmfCommand');

var _wmfCommand2 = _interopRequireDefault(_wmfCommand);

var colors = {
  red: new RGB(255, 0, 0)
};

/**
 * Hook to call before handling a specific command.
 *
 * @callback {WMFCommandManager~handleHook}
 * @param {WMFCommand} command - the command that will be executed
 * @param {Player} player - the exeucting player
 * @param {*} ...args - arguments
 * @returns {boolean} whether the command should be executed
 */

/**
 * The WMFCommandManager probes all packages for commands and registers them. It allows very customizable commands with automatic usage messages.
 */

var WMFCommandManager = (function (_WMFModule) {
  _inherits(WMFCommandManager, _WMFModule);

  /**
   * Creates the WMFCommandManager.
   */

  function WMFCommandManager() {
    var _this = this;

    _classCallCheck(this, WMFCommandManager);

    // super call must always be the directory name of this package.
    _get(Object.getPrototypeOf(WMFCommandManager.prototype), 'constructor', this).call(this, 'commands');

    /** @type {Map.<string, WMFCommand>} */
    this.commands = new Map();
    /** @type {Set.<WMFCommandManager~handleHook} */
    this.handleHooks = new Set();

    /** @type {Set.<string>} */
    this._registeredPackages = new Set();

    // Add the event handler
    events.Add('ChatCommand', function (player, command) {
      _this.handleCommand(player, command);
    });
  }

  /**
   * Initializes the package.
   */

  _createClass(WMFCommandManager, [{
    key: 'init',
    value: function init() {
      var _this2 = this;

      events.Add('WMFPackageInitialized', function (pkg) {
        return _this2._registerPackage(pkg);
      });
      _get(Object.getPrototypeOf(WMFCommandManager.prototype), 'init', this).call(this);
    }

    /**
     * Registers all commands of a package.
     *
     * @private
     * @param {WMFPackage} pkg - the wmf package to register
     */
  }, {
    key: '_registerPackage',
    value: function _registerPackage(pkg) {
      var _this3 = this;

      pkg.requestDirectory('commands').then(function (files) {
        files.forEach(function (commands) {
          commands.forEach(function (command) {
            return _this3.register(command);
          });
        });
      })['catch'](function (err) {
        if (!err.message.includes('ENOENT')) {
          log.error(err.stack);
        }
      });
    }

    /**
     * Handles a Command.
     *
     * @param {Player} player - executing player
     * @param {string} command - command string
     */
  }, {
    key: 'handleCommand',
    value: function handleCommand(player, command) {
      // parsing taken from the default package
      var args = command.match(/('(\\'|[^'])*'|"(\\"|[^"])*"|\/(\\\/|[^\/])*\/|(\\ |[^ ])+|[\w-]+)/g) || [];
      for (var idx = 1; idx < args.length; idx++) {
        if (args[idx].substr(0, 1) === '"' || args[idx].substr(0, 1) === "'") {
          args[idx] = JSON.parse(args[idx]);
        }
      }

      if (args.length === 0) {
        log.debug('invalid args length');
        return player.SendChatMessage('Unknown command \'' + command + '\'.', colors.red);
      }

      var commandName = args.splice(0, 1)[0];
      log.debug('WMFCommandManager#handleCommand \'' + commandName + '\'');
      if (!this.commands.has(commandName)) {
        return player.SendChatMessage('Unknown command \'' + command + '\'.', colors.red);
      }

      var cmd = this.commands.get(commandName);

      var rejectedByHook = false;
      this.handleHooks.forEach(function (hook) {
        if (!hook.apply(undefined, [cmd, player].concat(_toConsumableArray(args)))) {
          rejectedByHook = true;
        }
      });
      if (rejectedByHook) {
        log.debug('command execution of \'' + commandName + '\' blocked by a hook.');
      }
      cmd.execute(player, args);
    }

    /**
     * Registers a command
     *
     * @param {Object} info - command info
     * @param {Array.<string>} info.trigger - triggers for the command
     * @param {Array.<Array.<string>>} info.parameter - command parameter
     * @param {function(player: Player, ...args: *)} info.handler - handler
     */
  }, {
    key: 'register',
    value: function register(info) {
      var trigger = info.trigger;
      var handle = info.handle;

      delete info.trigger;
      delete info.handle;

      var command = new _wmfCommand2['default'](trigger, handle, info);
      for (var idx = 0; idx < trigger.length; idx++) {
        if (this.commands.has(trigger[idx])) {
          throw new Error('a command with the trigger \'' + trigger[idx] + '\' is already registered.');
        }

        this.commands.set(trigger[idx], command);
      }
      log.debug('command \'' + command.trigger.values().next().value + '\' registered: ' + command.usage);
    }
  }]);

  return WMFCommandManager;
})(_wmfModule2['default']);

exports['default'] = WMFCommandManager;
module.exports = exports['default'];
//# sourceMappingURL=main.js.map