/**
 * @overview WMF Permission Manager
 * @author Jan "Waffle" Christophersen
 * @copyright (c) Jan "Waffle" Christophersen
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

var _get = function get(_x, _x2, _x3) { var _again = true; _function: while (_again) { var object = _x, property = _x2, receiver = _x3; _again = false; if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { _x = parent; _x2 = property; _x3 = receiver; _again = true; desc = parent = undefined; continue _function; } } else if ('value' in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } } };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _wmfModule = require('../../wmfModule');

var _wmfModule2 = _interopRequireDefault(_wmfModule);

var _wmfPermission = require('./wmfPermission');

var _wmfPermission2 = _interopRequireDefault(_wmfPermission);

var _wmfPermissionContainer = require('./wmfPermissionContainer');

var _wmfPermissionContainer2 = _interopRequireDefault(_wmfPermissionContainer);

/**
 * The WMFPermissionManager manages all permissions. Permissions usually are just some names and an optional evaluator. Permissions make it easier to use less 'if (player.name === ...)' because you can simply add a permission to a player or remove it.
 */

var WMFPermissionManager = (function (_WMFModule) {
  _inherits(WMFPermissionManager, _WMFModule);

  /**
   * Creates the WMFPermissionManager.
   */

  function WMFPermissionManager() {
    var _this = this;

    _classCallCheck(this, WMFPermissionManager);

    // super call must always be the directory name of this package.
    _get(Object.getPrototypeOf(WMFPermissionManager.prototype), 'constructor', this).call(this, 'permissions');

    /** @type {Map.<string, WMFPermission>} */
    this.permissions = new Map();

    /** @type {Set.<string>} */
    this._registeredPackages = new Set();

    // Add a Permission Container to all new players
    events.Add('PlayerCreated', function (player) {
      player.permissions = _this.createContainer();
    });
  }

  /**
   * Initializes the package.
   */

  _createClass(WMFPermissionManager, [{
    key: 'init',
    value: function init() {
      var _this2 = this;

      // check if the commands module has been loaded already
      var cmd = this._wmf.packages.get('commands');
      if (typeof cmd !== 'undefined' && cmd.isModule) {
        cmd.handleHooks.add(this._commandHook.bind(this));
      } else {
        events.Add('WMFModuleInitialized', function (module) {
          if (module.name === 'commands') {
            module.handleHooks.add(_this2._commandHook.bind(_this2));
          }
        });
      }

      events.Add('WMFPackageInitialized', function (pkg) {
        return _this2._registerPackage(pkg);
      });
      _get(Object.getPrototypeOf(WMFPermissionManager.prototype), 'init', this).call(this);
    }

    /**
     * Looks up a WMFPermission.
     *
     * @param {string} name - permission name
     * @returns {WMFPermission|null}
     */
  }, {
    key: '_lookup',
    value: function _lookup(name) {
      if (this.permissions.has(name)) {
        return this.permissions.get(name);
      }
      return null;
    }

    /**
     * Registers all permissions of a package.
     *
     * @private
     * @param {WMFPackage} pkg - the wmf package to register
     */
  }, {
    key: '_registerPackage',
    value: function _registerPackage(pkg) {
      var _this3 = this;

      pkg.requestModule('permissions.js').then(function (permissions) {
        if (!(permissions instanceof Array)) {
          throw new Error('invalid permissions.js in package \'' + pkg.name + '\'');
        }

        permissions.forEach(function (permission) {
          if (typeof permission === 'string') {
            _this3.create(permission, {});
          } else if (typeof permission === 'object') {
            _this3.create(permission.name, permission.evaluate);
          }
        });
      })['catch'](function (err) {
        if (!err.message.includes('Cannot find module')) {
          log.error(err.stack);
        }
      });
    }

    /**
     * Hook for the command execution. Allows commands to have permissions.
     *
     * @private
     * @param {WMFCommand} command - the command that will be executed
     * @param {player} player - the executing player
     */
  }, {
    key: '_commandHook',
    value: function _commandHook(command, player) {
      if (typeof command.options.permissions === 'undefined') {
        return true;
      }
      if (typeof player.permissions === 'undefined') {
        return true;
      }

      log.debug('WMFPermissionManager#_commandHook');
      var permissions = command.options.permissions;

      for (var idx = 0; idx < permissions.length; idx++) {
        if (!player.permissions.has(permissions[idx])) {
          log.debug('WMFPermissionManager#_commandHook for player failed on permission \'' + permissions[idx] + '\'');
          player.SendChatMessage('You are not allowed to use the command.', new RGB(255, 0, 0));
          return false;
        }
      }
      return true;
    }

    /**
     * Creates a new Permission.
     *
     * @see {WMFPermission#constructor}
     * @param {string} name - permission name
     * @param {Object} [options] - options
     * @param {WMFPermission~evaluator} [options.evaluate] - evaluator
     */
  }, {
    key: 'create',
    value: function create(name, _ref) {
      var evaluate = _ref.evaluate;

      if (this.permissions.has(name)) {
        throw new Error('trying to redefine permission \'' + name + '\'');
      }
      this.permissions.set(name, new _wmfPermission2['default'](name, { evaluate: evaluate }));
    }

    /**
     * Creates a new Permission Container.
     *
     * @param {Array.<string>}
     * @returns {WMFPermissionContainer}
     */
  }, {
    key: 'createContainer',
    value: function createContainer() {
      return new _wmfPermissionContainer2['default'](this._lookup.bind(this));
    }
  }]);

  return WMFPermissionManager;
})(_wmfModule2['default']);

exports['default'] = WMFPermissionManager;
module.exports = exports['default'];
//# sourceMappingURL=main.js.map