/**
 * @overview WMF Permission Class
 * @author Jan "Waffle" Christophersen
 * @copyright (c) Jan "Waffle" Christophersen
 */

/**
 * Evaluates a Permission by the given handler. If the evaluator returns true, the permission will be held inside the container. Otherwise it will get removed. The evaluator's return value will be returned from WMFPermission#evaluate
 *
 * @callback {WMFPermission~evaluator}
 * @param {*} ...args - arguments
 * @returns {boolean}
 */

/**
 * WMFPermission is a simple representation of a permission. A permission usualy has a name. It is possible to also add an evaluation function in order to dynamically add or remove a permission.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var WMFPermission = (function () {
  /**
   * Creates a new Permission.
   *
   * @see {WMFPermission~evaluator}
   *
   * @param {string} name - permission name
   * @param {Object} [options] - options
   * @param {WMFPermission~evaluator} [options.evaluate] - evaluator
   */

  function WMFPermission(name, _ref) {
    var _ref$evaluator = _ref.evaluator;
    var evaluator = _ref$evaluator === undefined ? function () {
      return false;
    } : _ref$evaluator;

    _classCallCheck(this, WMFPermission);

    /** @type {string} */
    this.name = name;
    /** @type {WMFPermission~evaluator} */
    this.evaluator = evaluator;
  }

  /**
   * Evaluates the Permission. If no evaluator exists, 'false' will be returned.
   *
   * @param {*} ...args - arguments
   * @returns {boolean}
   */

  _createClass(WMFPermission, [{
    key: "evaluate",
    value: function evaluate() {
      return this.evaluator.apply(this, arguments);
    }
  }]);

  return WMFPermission;
})();

exports["default"] = WMFPermission;
module.exports = exports["default"];
//# sourceMappingURL=wmfPermission.js.map