/**
 * @overview WMF Wrapper Class
 * @author Jan "Waffle" Christophersen
 * @copyright (c) Jan "Waffle" Christophersen
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _wmfPackage = require('./wmfPackage');

var _wmfPackage2 = _interopRequireDefault(_wmfPackage);

var _wmfUtility = require('./wmfUtility');

var _wmfUtility2 = _interopRequireDefault(_wmfUtility);

/**
 * WMF Wrapper Class.
 */

var WMF = (function () {
  /**
   * Creates a new WMF Instance
   *
   * @param {Object} [options] - options
   * @param {string} [namespaceName=wmf] - global namespace name where all pacakages are available.
   */

  function WMF() {
    var _this = this;

    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, WMF);

    /** @type {Map.<string, Object>} */
    this.packages = new Map();
    /** @type {Map.<string, Object} */
    this.globals = new Map();

    /** @type {Object} */
    this.config = Object.assign({
      namespaceName: 'wmf',
      modules: []
    }, options);
    this.config.modules = new Set(this.config.modules);

    /** @type {Set.<string>} */
    this.availableModules = new Set();

    /** @type {Map.<string, Object>} */
    this.modules = new Map();

    _wmfUtility2['default'].readdir(_wmfUtility2['default'].path.join(__dirname, 'modules'), false).then(function (modules) {
      _this.availableModules = modules;
      _this._loadModules();
    })['catch'](function (err) {
      return log.error(err.stack);
    });
  }

  /**
   * Loads all available modules.
   */

  _createClass(WMF, [{
    key: '_loadModules',
    value: function _loadModules() {
      var _this2 = this;

      this.config.modules.forEach(function (moduleName) {
        if (!_this2.availableModules.has(moduleName)) {
          throw new Error('unknown module \'' + moduleName);
        }

        _this2.modules.set(moduleName, new (require(_wmfUtility2['default'].path.join(__dirname, 'modules', moduleName, 'main.js')))());
      });
    }

    /**
     * Returns the WMFPackage class.
     *
     * @type {WMFPackage}
     */
  }, {
    key: 'Package',
    get: function get() {
      return _wmfPackage2['default'];
    }

    /**
     * Returns the WMFUtility class.
     *
     * @type {WMFUtility}
     */
  }, {
    key: 'Utility',
    get: function get() {
      return _wmfUtility2['default'];
    }
  }]);

  return WMF;
})();

exports['default'] = WMF;
module.exports = exports['default'];
//# sourceMappingURL=wmf.js.map