/**
 * @overview WMF Module Class
 * @author Jan "Waffle" Christophersen
 * @copyright (c) Jan "Waffle" Christophersen
 */

'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

var _get = function get(_x2, _x3, _x4) { var _again = true; _function: while (_again) { var object = _x2, property = _x3, receiver = _x4; _again = false; if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { _x2 = parent; _x3 = property; _x4 = receiver; _again = true; desc = parent = undefined; continue _function; } } else if ('value' in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } } };

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _wmfPackage = require('./wmfPackage');

var _wmfPackage2 = _interopRequireDefault(_wmfPackage);

var _wmfUtility = require('./wmfUtility');

var _wmfUtility2 = _interopRequireDefault(_wmfUtility);

/**
 * The WMFModule class is used as a superset of WMFPackage to allow internal modules.
 * It can be safely assumed that required modules are loaded before any package and thus use the WMFPackageInitialized event
 * to execute functionality when a package is being loaded.
 */

var WMFModule = (function (_WMFPackage) {
  _inherits(WMFModule, _WMFPackage);

  /**
   * Creates a new WMFModule
   */

  function WMFModule(name) {
    _classCallCheck(this, WMFModule);

    _get(Object.getPrototypeOf(WMFModule.prototype), 'constructor', this).call(this, name, true);

    /** @type {boolean} */
    this.initialized = false;
  }

  /**
   * Initializes the module.
   */

  _createClass(WMFModule, [{
    key: 'init',
    value: function init() {
      this.initialized = true;
      log.info('module \'' + this.name + '\' has been initialized.');
      events.Call('WMFModuleInitialized', this);
    }

    /**
     * Requests a File (module, either *.js or *.json) from the package's directory. The name of the WMFPackage needs to be set to the name of the folder the package resides in.
     *
     * @param {string} file - path to the file
     * @returns {Promise.<Object>}
     */
  }, {
    key: 'requestModule',
    value: function requestModule(file) {
      var _this = this;

      return new Promise(function (resolve, reject) {
        if (!_wmfUtility2['default'].path.extname(file).startsWith('.js')) {
          return reject(new Error('only *.js and *.json files are allowed.'));
        }

        var dir = file;
        if (_wmfUtility2['default'].path.isAbsolute(dir)) {
          dir = _wmfUtility2['default'].path.relative(__dirname + '/modules/' + _this.name + '/', dir);
        }
        resolve(require(__dirname + '/modules/' + _this.name + '/' + dir));
      });
    }

    /**
     * Requests all modules of a directory (*.js or *.json files). The name of the WMFPackage needs to be set to the name of the folder the package resides in.
     *
     * @param {string} directory - path to the directory
     * @param {boolean} [recursive=true] - include subdirectories
     * @returns {Promise.<Map.<Object>>}
     */
  }, {
    key: 'requestDirectory',
    value: function requestDirectory(directory) {
      var _this2 = this;

      var recursive = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];

      return new Promise(function (resolve, reject) {
        var dir = directory;
        if (_wmfUtility2['default'].path.isAbsolute(dir)) {
          dir = _wmfUtility2['default'].path.relative(__dirname + '/modules/' + _this2.name + '/', dir);
        }
        dir = __dirname + '/modules/' + _this2.name + '/' + dir;

        _wmfUtility2['default'].readdir(dir, recursive).then(function (files) {
          var promises = [];
          files.forEach(function (file) {
            if (!_wmfUtility2['default'].path.extname(file).startsWith('.js')) {
              return undefined;
            }

            promises.push(_this2.requestModule(_wmfUtility2['default'].path.join(directory, file)).then(function (mod) {
              return [file, mod];
            }));
          });

          Promise.all(promises).then(function (modules) {
            resolve(new Map(modules));
          })['catch'](reject);
        })['catch'](reject);
      });
    }
  }]);

  return WMFModule;
})(_wmfPackage2['default']);

exports['default'] = WMFModule;
module.exports = exports['default'];
//# sourceMappingURL=wmfModule.js.map